import java.awt.Graphics;
import java.io.*;

/**
 * Noppa - pelin perusyksikk
 */
public class Die implements Serializable {
    // Y-koordinaatti animointia varten
    private float y;
    private float ySpeed;

    // Nopan arvo: 1-6, JOKER=jokeri
    private int value;

    // Nopan tila: oma (OWN), vastustajan (OPPONENT), vapaa (AVAILABLE)
    private int status;

    /**
     * Pelaaja valinnut
     */
    public final static int OWN = 0;

    /**
     * Vastustaja valinnut
     */
    public final static int OPPONENT = 1;

    /**
     * Vapaa noppa
     */
    public final static int AVAILABLE = 2;

    /**
     * Jokerinoppa
     */
    public final static int JOKER = 7;

    /**
     * Erilaisten noppien mr
     */
    public final static int NUMBER_OF_VALUES = 7;
	    
    private final static float GRAVITY = 0.05f;
    
    /**
     * Alustaa nopan annetuilla arvoilla
     * @param y Nopan y-koordinaatti
     * @param value Nopan arvo
     */
    public Die(int y, int value) {
	this.y = y;
	this.value = value;
	status = AVAILABLE;
        ySpeed = 0;
    }
    
    /**
     * Antaa nopan Y-koordinaatin
     * @return y-koordinaatti
     */
    public float getY() {
	return y;
    }

    /**
     * Antaa nopan arvon
     * @return nopan arvo
     */
    public int getValue() {
	return value;
    }

    /**
     * Antaa nopan tilan
     * @return nopan tila
     */
    public int getStatus() {
	return status;
    }

    /**
     * Katsoo onko pelaaja jo valinnut nopan
     * @return onko pelaaja valinnut nopan
     */
    public boolean isSelected() {
	return status == OWN;
    }
    
    /**
     * Asettaa y-koordinaatin
     * @param y nopan y-koordinaatti
     */
    public void setY(float y) {
	this.y=y;
    }

    /**
     * Asettaa nopan arvon
     * @param value nopan arvo
     */
    public void setValue(int value) {
	this.value=value;
    }

    /**
     * Asettaa nopan tilan
     * @param status nopan tila
     */
    public void setStatus(int status) {
	this.status=status;
    }

    /**
     * Muuttaa nopan tilan vapaaksi jos valittu ja pinvastoin
     */
    public void changeStatus() {
	if(this.status == OWN)
	    this.status = AVAILABLE;
	else
	    if(this.status == AVAILABLE)
		this.status = OWN;
    }

    /**
     * Muuttaa nopan tilan vapaaksi jos vastustaja on valinnut ja pin vastoin
     */
    public void changeOppStatus() {
	if(this.status == OPPONENT)
	    this.status = AVAILABLE;
	else
	    if(this.status == AVAILABLE)
		this.status = OPPONENT;
    }

    /**
     * Muuttaa nopan y-koordinaattia animaatiota varten
     * @param y y-koordinaatti, jota kohti noppa putoaa
     */
    public void animateTo(float y) {
	float deltay = Math.abs(this.y - y);
	    
	if( deltay > 0 ) {
	    ySpeed += GRAVITY;
	    if( deltay < ySpeed ) {
		this.y = y;
		ySpeed = 0;
	    }
	    else {
		this.y += ySpeed;
	    }
	}
	else {
	    ySpeed = 0;
	}
    }
    
    /**
     * Piirt nopan
     * @param x nopan sarakkeen x-koordinaatti
     * @param g Graphics-objekti piirtoa varten
     * @param r Resource-instanssi, josta nopan kuva ja sen koko haetaan
     * @param imageObserver objekti, jolle ilmoitetaan kuvan piirtmisen edistymisest
     */
    public void draw( float x, Graphics g, Resource r, Game imageObserver ) {
        try {
            if( status == AVAILABLE) {
                g.drawImage( r.getDice(value - 1), 32+(int)(x*r.DICEWIDTH), 50+(int)(y*r.DICEHEIGHT), imageObserver );
            }
            else if( status == OWN) {
                g.drawImage( r.getOwnDice(value - 1), 32+(int)(x*r.DICEWIDTH), 50+(int)(y*r.DICEHEIGHT), imageObserver );
            }
            else if( status == OPPONENT) {
                g.drawImage( r.getOppDice(value - 1), 32+(int)(x*r.DICEWIDTH), 50+(int)(y*r.DICEHEIGHT), imageObserver );
            }
        } catch(Exception e) {
        }
    }
    

}
