import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;

public class Game extends JPanel implements MouseListener {
    
    private GameData gd;
    private Resource resource;
    private serverManager server;

    private static final int DICEWIDTH = 32;
    private static final int DICEHEIGHT = 32;

    /**
     * Alustaa pelin ja liitt Resourcen
     * @param resource liitettv Resourcen instanssi
     */
    public Game( Resource resource ) {
	this.gd = null;
	this.resource = resource;
    }

    /**
     * Aloittaa moninpelin
     */    
    public GameData startMultiplayerGame() {
	if(!resource.isClient()){
	gd = new GameData( resource.getWidth(), resource.getHeight() );
	Server apuserver = new Server(gd);
	socketServer itse = new socketServer(resource.getPort(),apuserver);
	server = new serverManager(true,this);
			apuserver.setRemote(itse);
			apuserver.setLocal(server);
			server.setServ(apuserver);
			itse.getClient();
    	}
    	else{
    		server = new serverManager(false,this);
		socketClient cli = new socketClient();
		server.setNet(cli);
		cli.setLocal(server);
		cli.connect(resource.getIp(),resource.getPort());
		server.getGameData();
		
	}
	
	// wait while transfering gamedata
	while( gd == null ) { 
		try { Thread.sleep(1000); } catch (Exception e) {}
	} 
	
	resource.SetSize( gd.getWidth(), gd.getHeight() );
	
	return gd;
    }
    
    /**
     * Piirt kentn
     * @param g Graphics-objekti piirtoa varten
     */
    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	    
        // (woody) background
        g.drawImage(resource.getBackground(), 0, 0, this);

        // scores etc ...
        g.setColor(Color.white);
	g.drawString("Your Score:" + gd.getOwnScore(), 34, 37);
	g.drawString("Opponent Score:"+gd.getOppScore(), 32+(gd.getWidth()-4)*32+2 /*548*/ ,37);
        // dice
	for(int i=0; i < gd.getWidth(); i++) {
            gd.getColumn(i).animateTo(i);
            gd.getColumn(i).draw(g, resource, this);
	}
    }

    /**
     * Kertoo, voiko kyseisen nopan valinnan poistaa
     * Nopan valinnan poistaminen saattaa rikkoa valittujen noppien alueen. Tt metodia kytetn tarkistamaan, ett nin ei ky.     
     * @param x nopan looginen x-koordinaatti
     * @param y nopan looginen y-koordinaatti
     * @return voiko nopan valinnan poistaa sntjen mukaan
     */
    private boolean isUnselectable(int x, int y) {
	final int WEST = 0;
	final int NORTH = 1;
	final int EAST = 2;
	final int SOUTH = 3;
	int numNeighbours = 0;
	Die neighbours[] = new Die[4];
	neighbours[WEST] = gd.getDie(x-1, y);
	neighbours[NORTH] = gd.getDie(x, y-1);
	neighbours[EAST] = gd.getDie(x+1, y);
	neighbours[SOUTH] = gd.getDie(x, y+1);
	boolean own[] = new boolean[4];
	
	for (int i = 0; i < 4; i++) {
	    if (neighbours[i].isSelected()) {
		numNeighbours++;
		own[i] = true;
	    }
	    else
		own[i] = false;
	}
	
	
	if (numNeighbours < 2)
	    return true;
	else if (numNeighbours == 4)
	    return false;
	else if (numNeighbours == 2) {
	    if ((own[NORTH] && own[SOUTH]) || (own[WEST] && own[EAST]))
		return false;
	    else if (own[WEST] && own[NORTH])
		return gd.getDie(x-1, y-1).isSelected();
	    else if (own[NORTH] && own[EAST])
		return gd.getDie(x+1, y-1).isSelected();
	    else if (own[EAST] && own[SOUTH])
		return gd.getDie(x+1, y+1).isSelected();
	    else
		return gd.getDie(x-1, y+1).isSelected();
	}
	else {
	    if (!own[WEST])
		return (gd.getDie(x+1, y-1).isSelected() && 
			gd.getDie(x+1, y+1).isSelected());	
	    else if (!own[NORTH])
		return (gd.getDie(x-1, y+1).isSelected() && 
			gd.getDie(x+1, y+1).isSelected());
	    else if (!own[EAST])
		return (gd.getDie(x-1, y-1).isSelected() && 
			gd.getDie(x-1, y+1).isSelected());
	    else
		return (gd.getDie(x-1, y-1).isSelected() && 
			gd.getDie(x+1, y-1).isSelected());
	}
    }

    /**
     * Kertoo, voiko kyseisen nopan valita
     * Jos noppa ei ole kiinni jo valittujen alueessa (tai ensimminen valittava), sit ei voi valita.
     * @param x nopan looginen x-koordinaatti
     * @param y nopan looginen y-koordinaatti
     * @return voiko nopan valita
     */
    boolean isSelectable(int x, int y) {
        if( gd.getDie(x,y).getStatus() == Die.AVAILABLE ) {
            if(gd.getOwnPoints()==0) { // we haven't selected any dice yet, anything goes this time
                return true;
            }
            
            if(gd.getDie((x-1),y).getStatus() == Die.OWN // do we have neightbours?
                || gd.getDie((x+1),y).getStatus() == Die.OWN
                || gd.getDie(x,(y-1)).getStatus() == Die.OWN
		|| gd.getDie(x,(y+1)).getStatus() == Die.OWN) {
                return true;
            }
        }
        
        return false;
    }
       
    /**
     * Ksittelee hiiren klikkaukset
     * Katsoo, mihin kohtaan pelikentt on klikattu ja toimii sen mukaisesti (valitsee nopan/poistaa nopan valinnan jne).
     * @param e tiedot hiiren klikkauksesta
     */
    public void mouseClicked(MouseEvent e) {
	if(e.getX() > 32 && e.getX() < 32+gd.getWidth()*DICEWIDTH && e.getY() > 50 && e.getY() < 50+gd.getHeight()*DICEHEIGHT) {  
	    int x, y;
	    x = Math.abs((e.getX() - 32) / DICEWIDTH);
	    y = Math.abs((e.getY() - 50) / DICEHEIGHT);

	    int valueOfDie = gd.getDie(x,y).getValue();
            
            if( valueOfDie == 0 ) { // there is no die
                return;
            }
            
            if(gd.getDie(x,y).getStatus() == Die.OWN) { // we clicked our own die
                if(isUnselectable(x,y)) {               // can we unselect it?
                    //gd.getDie(x,y).changeStatus();
                    //gd.addOwnPoints(-valueOfDie);
                 
                    server.pickDie(x,y);
                }
                return;
            }
            
            if(isSelectable(x,y)) {
            	if(valueOfDie == Die.JOKER ||
            		(valueOfDie + gd.getOwnPoints()) < 7 ||
            		(valueOfDie + gd.getOwnPoints()) == 7)	
            		server.pickDie(x,y);
                
            }  
        }
    }

    /**
     * Viesti serverilt, ett noppa on valittu
     * @param x nopan looginen x-koordinaatti
     * @param y nopan looginen y-koordinaatti
     * @param player pelaaja, joka on valinnut nopan
     */    
    public void dieSelected(int x,int y,int player){
    	if(player==Die.OWN){
    		int valueOfDie = gd.getDie(x,y).getValue();
    		if(gd.getDie(x,y).getStatus() == Die.OWN) {
                	
                    		gd.getDie(x,y).changeStatus();
                    		gd.addOwnPoints(-valueOfDie);
                	return;
        	}
	    	
	    	gd.getDie(x,y).changeStatus();
	    	if(valueOfDie == Die.JOKER){
	    		//gd.removeDice(Die.OWN);
	    		server.removeMine();
	    		gd.addOwnPoints(-gd.getOwnPoints());
	    		}
	    	else if((valueOfDie + gd.getOwnPoints()) < 7 ){
	    		gd.addOwnPoints(valueOfDie);
	    	}
	    	else if((valueOfDie + gd.getOwnPoints()) == 7){
	    		gd.addOwnPoints(valueOfDie);
	    		//gd.removeDice(Die.OWN);
	    		server.removeMine();
	    		gd.addOwnPoints(-7);
	    	}
	}
	else gd.getDie(x,y).changeOppStatus();
  }
  
    /**
     * Poistaa toisen pelaajan valitsemat nopat
     * @param player pelaaja, jonka valitsemat nopat poistetaan
     */
    public void removeDice(int player){
  	gd.removeDice(player);
    }

    /**
     * Asettaa kytettvn pelidatan
     * @param x kytettv pelidata
     */
    public void setGameData(GameData x){
    	gd=x;
    }

    // Muut hiiritapahtumat eivt johda toimenpiteisiin
    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

}

