import java.util.*;
import java.io.*;

public class GameData implements Serializable {
    private int ownPoints = 0; //noppien tmnhetkinen summa

    private int ownScore = 0;
    private int oppScore = 0;

    private int height = 0;
    private int width = 0;

    // Lista noppasarakkeista
    private List<Column> listOfColumns = new ArrayList<Column>();

    /**
     * Luo m x n kokoisen pelikentn
     * @param m kentn leveys
     * @param n kentn korkeus
     */
    public GameData(int m, int n) {
	this.width = m;
	this.height = n;
	for (int i=0; i<m; i++) {
	    listOfColumns.add(new Column(i, n));
        }
    }

    /**
     * Palauttaa nopan
     * @param x nopan looginen x-koordinaatti
     * @param y nopan looginen y-koordinaatti
     */
    public Die getDie(int x, int y) {
	if((x >= 0 && x <= (width-1)) && (y >= 0 && y <= (height-1))) {
            return listOfColumns.get(x).getDie(y);
        } 
	else { 
	    return new Die(y,0);
        }
    }

    /**
     * Palauttaa sarakkeen
     * @param x sarakkeen looginen x-koordinaatti
     * @return sarake
     */
    public Column getColumn(int x) {
	return listOfColumns.get(x);
    }

    /**
     * Kasvattaa valittujen noppien summaa
     * @param value mr, jolla summaa kasvatetaan
     */
    public void addOwnPoints(int value) {
	ownPoints += value;
    }

    /**
     * Palauttaa omat pisteet
     * @return omat pisteet
     */
    public int getOwnScore() {
	return ownScore;
    }

    /**
     * Palauttaa vastustajan pisteet
     * @return vastustajan pisteet
     */    
    public int getOppScore() {
    	return oppScore;
    }

    /**
     * Palauttaa valittujen noppien summan
     * @return valittujen noppien summa
     */
    public int getOwnPoints() {
	return ownPoints;
    }

    /**
     * Palauttaa sarakkeen korkeuden
     * @return sarakkeen korkeus
     */
    public int getHeight() {
	return height;
    }

    /**
     * Palauttaa sarakkeen leveyden
     * @return sarakkeen leveys
     */   
    public int getWidth() {
	return width;
    }

    /**
     * Poistaa nopat, joilla tietty tila
     * @param status tila, jossa olevat nopat poistetaan
     */
    public void removeDice(int status) {
	for(int i=0; i < width; i++) {
	    boolean dieRemoved = false;
	    for(int j=0; j < height; j++)
		if(getDie(i,j).getStatus() == status) {
		    getColumn(i).removeDie(j);
		    if(status==Die.OWN) ownScore++;
		    else oppScore++;
		    if (!dieRemoved)
			dieRemoved = true;
		}

	    if(dieRemoved && getColumn(i).getDie(height-1).getValue() == 0) {
		listOfColumns.remove(i);
		if(i < (width/2))
		    listOfColumns.add(0, new Column(0, height, 0));
		if(i >= (width/2)) {
		    listOfColumns.add(new Column(0, height, 0));
		    i--; // jotta ei hypt yht saraketta yli		    
		}
	    }
	}
    }
    
}
