import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;

public class Resource {
    
    public final static int DICEWIDTH = 32;
    public final static int DICEHEIGHT = 32;
    
    private BufferedImage dieImage[] = new BufferedImage[Die.NUMBER_OF_VALUES];
    private BufferedImage dieImageOwn[] = new BufferedImage[Die.NUMBER_OF_VALUES];
    private BufferedImage dieImageOpp[] = new BufferedImage[Die.NUMBER_OF_VALUES];
    private BufferedImage back;
    
    private boolean loaded = false; 

    private String ipAddress;
    private int portNumber;
    private int width;
    private int height;
    private boolean client;
	    
    private boolean setupDone = false;

    public Resource() { }

    /**
     * Lataa kuvat
     */
    public void Load() {
	if(!loaded) {
	    loaded = true;

	    try {
                File file;
                
		for(int i=0; i < Die.NUMBER_OF_VALUES; i++) {
                    file = new File("data/noppa" + (i+1) + ".gif");
		    dieImage[i] = ImageIO.read(file);
		    
		    file = new File("data/noppa" + (i+1) + "oma.gif");
		    dieImageOwn[i] = ImageIO.read(file);
		    
		    file = new File("data/noppa" + (i+1) + "vas.gif");
		    dieImageOpp[i] = ImageIO.read(file);
		}	
                
                if (width == 12) {
		    file = new File("data/12x8.gif");
		    back = ImageIO.read(file);
                } else if (width == 18) {
		    file = new File("data/18x12.gif");
		    back = ImageIO.read(file);
                } else if (width == 24) {
		    file = new File("data/24x16.gif");
		    back = ImageIO.read(file);
		} else {
		    System.out.println("joo: "+width);
		}
			
		
		
	    } catch (Exception e) {
	    }
	}

    }

    /**
     * Palauttaa valitsemattoman nopan kuvan
     * @param index kuvan indeksi
     * @return nopan kuva
     */
    public BufferedImage getDice(int index) {
	return dieImage[index];
    }

    /**
     * Palauttaa pelaajan itse valitseman nopan kuvan
     * @param index kuvan indeksi
     * @return nopan kuva
     */
    public BufferedImage getOwnDice(int index) {
        return dieImageOwn[index];
    }

    /**
     * Palauttaa vastustajan valitseman nopan kuvan
     * @param index kuvan indeksi
     * @return nopan kuva
     */
    public BufferedImage getOppDice(int index) {
        return dieImageOpp[index];
    }
    
    /**
     * Palauttaa taustakuvan
     * @return taustakuva
     */
    public BufferedImage getBackground() {
        return back;
    }

    /**
     * Asettaa asiakkaan asetukset
     * @param ip IP-osoite
     * @param port porttinumero
     */
    public void setClientOptions(String ip, int port) {
	client = true;
	ipAddress = ip;
	portNumber = port;
	setupDone = true;
    }

    /**
     * Asettaa palvelimen asetukset
     * @param width kentn leveys
     * @param height kentn korkeus
     * @param port porttinumero
     */
    public void setServerOptions(int width, int height, int port) {
	client = false;
	this.height = height;
	this.width = width;
	portNumber = port;
	setupDone = true;
    }

    /**
     * Palauttaa portin numeron
     * @return porttinumero
     */
    public int getPort() {
	return portNumber;
    }
 
    /**
     * Palauttaa IP-osoitteen
     * @return IP-osoite
     */
    public String getIp(){
    	return ipAddress;
    }

    /**
     * Kertoo, onko pelaaja asiakas
     * @return onko asiakas
     */
    public boolean isClient(){
    	return client;
    }
    
    /**
     * Asettaa kentn koon
     * @param w kentn leveys
     * @param h kentn korkeus
     */
    public void SetSize(int w, int h) {
	height = h;
	width = w;	    
    }
    
    /**
     * Palauttaa kentn leveyden
     * @return kentn leveys
     */
    public int getWidth() {
	return width;
    }
    
    /**
     * Palauttaa kentn korkeuden
     * @return kentn korkeus
     */
    public int getHeight() {
	return height;
    }

    /**
     * Kertoo, onko asetukset suoritettu
     * @return onko asetukset suoritettu
     */    
    public boolean isSetupDone() {
	return setupDone;
    }
}
