import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import java.net.*;

/**
 * Aloitusruudun nyttminen ja toiminnot
 */
public class StartScreen {
    // Ikkunan pkehys
    private final JFrame mainFrame = new JFrame("Seven: settings");
    
    // Asetuksien tallennukseen
    private Resource resource;

    // Merkkijonovakiot
    final String csDescription = "Client/server";
    final String ipDescription = "IP address";
    final String portDescription = "Port number";
    final String sizeDescription = "Game area size";
    final String clientDescription = "Client";
    final String serverDescription = "Server";
    final String joinDescription = "Join";
    final String hostDescription = "Host";
    final String sizeDescriptions[] = {"12x8", "18x12", "24x16"};
    final int numSizes = 3;

    // Kyttliittymn komponentit
    private final ButtonGroup csGroup = new ButtonGroup();
    private final ButtonGroup sizeGroup = new ButtonGroup();
    private JButton connectButton;
    private JFormattedTextField ipTextFields[];
    private JFormattedTextField portTextField;

    /** 
     * Alustaa kyttliittymn komponentit
     */
    public StartScreen() {


	connectButton = new JButton(joinDescription);

	ipTextFields = new JFormattedTextField[4];
	for (int i = 0; i < ipTextFields.length; i++)
	  ipTextFields[i] = new JFormattedTextField(new DecimalFormat("0"));

	portTextField = new JFormattedTextField(new DecimalFormat("0"));
    }

    // Kyttliittymn eri osat omina paneeleinaan, jotta tarvittaessa niit
    // voi muotoilla ja sijoitella paremmin

    /**
     * Luo paneelin, jossa on asiakas/palvelin radiobuttonit 
     * @return asiakas/palvelin-paneeli
     */
    private JPanel createClientServerPanel() {
	final int numRadioButtons = 2;
	
	JPanel csPanel = new JPanel();

	JLabel csLabel = new JLabel(csDescription);
	JRadioButton csRadioButtons[] = new JRadioButton[numRadioButtons];

	csRadioButtons[0] = new JRadioButton(clientDescription);
	csRadioButtons[0].setActionCommand(clientDescription);
	csRadioButtons[0].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    clientSelected(true);
		}
	    });

	csRadioButtons[1] = new JRadioButton(serverDescription);
	csRadioButtons[1].setActionCommand(serverDescription);
	csRadioButtons[1].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    clientSelected(false);
		}
	    });

	for (int i = 0; i < numRadioButtons; i++)
	    csGroup.add(csRadioButtons[i]);

	csRadioButtons[0].setSelected(true);

        csPanel.add(csLabel);

        for (int i = 0; i < numRadioButtons; i++)
            csPanel.add(csRadioButtons[i]);

	return csPanel;
    }

    /**
     * Luo paneelin, jossa yhdist-painike
     * @return yhdist-paneeli
     */
    private JPanel createConnectPanel() {
	JPanel connectPanel = new JPanel();

	connectButton.setPreferredSize(new Dimension(100, 40));
	connectButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    connectButtonPressed();
		}
	    });

	connectPanel.add(connectButton);

	return connectPanel;
    }

    /**
     * Luo paneelin, jossa tekstikentt porttinumeroa varten
     * @return portti-paneeli
     */
    private JPanel createPortPanel () {
	JPanel portPanel = new JPanel();

	JLabel portLabel = new JLabel(portDescription);

	portTextField.setText("9000");
	portTextField.setColumns(4);
	
	portPanel.add(portLabel);
	portPanel.add(portTextField);

	return portPanel;
    }

    /**
     * Luo paneelin, jossa tekstikentt IP-osoitetta varten
     * @return IP-paneeli
     */
    private JPanel createIPPanel() {
	JPanel ipPanel = new JPanel();

	JLabel ipLabel = new JLabel(ipDescription);

        ipPanel.add(ipLabel);

	for (int i = 0; i < ipTextFields.length; i++) {
	    ipTextFields[i].setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
	    ipTextFields[i].setSize(100, 20);
	    ipTextFields[i].setColumns(3);
	    ipTextFields[i].setText("0");
	    ipPanel.add(ipTextFields[i]);
	}

	return ipPanel;
    }

    /**
     * Luo paneelin, jossa valitaan radiobuttoneilla kentn koko
     * @return kentn koko-paneeli
     */
    private JPanel createSizePanel() {
	JPanel serverPanel = new JPanel();

	JLabel serverLabel = new JLabel(sizeDescription);
	JRadioButton radioButtons[] = new JRadioButton[numSizes];

	for (int i = 0; i < numSizes; i++) {
	    radioButtons[i] = new JRadioButton(sizeDescriptions[i]);
	    radioButtons[i].setActionCommand(sizeDescriptions[i]);
	}

	for (int i = 0; i < numSizes; i++)
	    sizeGroup.add(radioButtons[i]);

	radioButtons[0].setSelected(true);

        serverPanel.add(serverLabel);

        for (int i = 0; i < numSizes; i++)
            serverPanel.add(radioButtons[i]);

	return serverPanel;
    }

    /** 
     * Luo ja nytt kyttliittymn
     * Kasaa paneelit kokonaisuudeksi vakiokokoiseen ikkunaan ja nytt sen.
     */
    private void createAndShowGUI() {
	// Ikkunan asetukset
	mainFrame.setResizable(false);
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	// Lis paneelit
        Container contentPane = mainFrame.getContentPane();
        contentPane.setLayout(new GridLayout(5, 1, 0, 20));
        contentPane.add(createClientServerPanel());
	contentPane.add(createConnectPanel());
	contentPane.add(createPortPanel());
	contentPane.add(createIPPanel());
	contentPane.add(createSizePanel());

	// Oikeat vaihtoehdot valittaviksi
	clientSelected(true);

        // Nytt ikkunan
        mainFrame.pack();
        mainFrame.setVisible(true);
    }


    /**
     * Muuttaa komponenttien tilaa sen mukaan, onko palvelin vai asiakas valittu
     * @param t onko asiakas valittu
     */
    private void clientSelected(boolean t) {
	// IP-osoitekentt kyttn vain asiakkaalle
	for (int i = 0; i < ipTextFields.length; i++)
	    ipTextFields[i].setEnabled(t);

	// Kentn koon valinta palvelinpuolella
	for (Enumeration e = sizeGroup.getElements(); e.hasMoreElements();) {
	    ((JRadioButton) e.nextElement()).setEnabled(!t);
	}

	// Yhdist-painikkeen teksti
	if (t)
	    connectButton.setText("Join");
	else
	    connectButton.setText("Host");
    }


    /**
     * Yhdist-painikkeen toiminta
     * Tallentaa sytetyt tiedot annettuun Resourceen ja sulkee alkuruudun.
     */
    private void connectButtonPressed() {
	String cs;
	int port;

	// Onko palvelin vai asiakas valittu
	cs = csGroup.getSelection().getActionCommand();

	// Lukee porttinumeron
	port = Integer.parseInt(portTextField.getText());

	// Jos asiakas valittu
	if (cs.equals(clientDescription)) {
	    String[] ip = new String[4];
	    String ipAddress = new String();

	    // Lukee IP-osoitteen
	    for (int i = 0; i < 4; i++) {
		ip[i] = ipTextFields[i].getText();
		if (i>0)
		    ipAddress += ".";
		ipAddress += ip[i];
	    }

	    // Tallentaa asetukset Resourceen
	    resource.setClientOptions(ipAddress, port);
	}
	else { // Jos palvelin valittu
	    String size;
	    int width, height;

	    // Lukee valitun koon
	    size = sizeGroup.getSelection().getActionCommand();

	    // Muuttaa leveyden ja korkeuden vastaamaan valittua kokovaihtoehtoa
	    if (size.equals(sizeDescriptions[0])) {
		width = 12;
		height = 8;
	    }
	    else if (size.equals(sizeDescriptions[1])) {
		width = 18;
		height = 12;
	    }
	    else {
		width = 24;
		height = 16;
	    }

	    // Tallentaa asetukset Resourceen
	    resource.setServerOptions(width, height, port);
	}

	mainFrame.dispose();
    }

    /**
     * Laittaa kyttliittymn toimintaan
     * @param r Resource-luokan instanssi, jonne asetukset tallennetaan
     */
    public void run(Resource r) {
	resource = r;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
